# Copyright (C) 2015-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from rest_framework.test import APITestCase
from unittest.mock import patch

from swh.web.common.exc import NotFoundExc
from swh.web.api.views.revision import (
    _revision_directory_by
)

from swh.web.tests.testcase import WebTestCase


class RevisionApiTestCase(WebTestCase, APITestCase):

    @patch('swh.web.api.views.revision.service')
    def test_api_revision(self, mock_service):
        # given
        stub_revision = {
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'author_name': 'Software Heritage',
            'author_email': 'robot@softwareheritage.org',
            'committer_name': 'Software Heritage',
            'committer_email': 'robot@softwareheritage.org',
            'message': 'synthetic revision message',
            'date_offset': 0,
            'committer_date_offset': 0,
            'parents': ['8734ef7e7c357ce2af928115c6c6a42b7e2a44e7'],
            'type': 'tar',
            'synthetic': True,
            'metadata': {
                'original_artifact': [{
                    'archive_type': 'tar',
                    'name': 'webbase-5.7.0.tar.gz',
                    'sha1': '147f73f369733d088b7a6fa9c4e0273dcd3c7ccd',
                    'sha1_git': '6a15ea8b881069adedf11feceec35588f2cfe8f1',
                    'sha256': '401d0df797110bea805d358b85bcc1ced29549d3d73f'
                    '309d36484e7edf7bb912'
                }]
            },
        }
        mock_service.lookup_revision.return_value = stub_revision

        expected_revision = {
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'url': '/api/1/revision/18d8be353ed3480476f032475e7c233eff7371d5/',
            'history_url': '/api/1/revision/18d8be353ed3480476f032475e7c233e'
                           'ff7371d5/log/',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'directory_url': '/api/1/directory/7834ef7e7c357ce2af928115c6c6'
                             'a42b7e2a44e6/',
            'author_name': 'Software Heritage',
            'author_email': 'robot@softwareheritage.org',
            'committer_name': 'Software Heritage',
            'committer_email': 'robot@softwareheritage.org',
            'message': 'synthetic revision message',
            'date_offset': 0,
            'committer_date_offset': 0,
            'parents': [{
                'id': '8734ef7e7c357ce2af928115c6c6a42b7e2a44e7',
                 'url': '/api/1/revision/8734ef7e7c357ce2af928115c6c6a42b7e2a44e7/'  # noqa
            }],
            'type': 'tar',
            'synthetic': True,
            'metadata': {
                'original_artifact': [{
                    'archive_type': 'tar',
                    'name': 'webbase-5.7.0.tar.gz',
                    'sha1': '147f73f369733d088b7a6fa9c4e0273dcd3c7ccd',
                    'sha1_git': '6a15ea8b881069adedf11feceec35588f2cfe8f1',
                    'sha256': '401d0df797110bea805d358b85bcc1ced29549d3d73f'
                    '309d36484e7edf7bb912'
                }]
            },
        }

        # when
        rv = self.client.get('/api/1/revision/'
                             '18d8be353ed3480476f032475e7c233eff7371d5/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(expected_revision, rv.data)

        mock_service.lookup_revision.assert_called_once_with(
            '18d8be353ed3480476f032475e7c233eff7371d5')

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_not_found(self, mock_service):
        # given
        mock_service.lookup_revision.return_value = None

        # when
        rv = self.client.get('/api/1/revision/12345/')

        # then
        self.assertEqual(rv.status_code, 404)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, {
            'exception': 'NotFoundExc',
            'reason': 'Revision with sha1_git 12345 not found.'})

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_raw_ok(self, mock_service):
        # given
        stub_revision = {'message': 'synthetic revision message'}

        mock_service.lookup_revision_message.return_value = stub_revision

        # when
        rv = self.client.get('/api/1/revision/18d8be353ed3480476f032475e7c2'
                             '33eff7371d5/raw/')
        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/octet-stream')
        self.assertEqual(rv.content, b'synthetic revision message')

        mock_service.lookup_revision_message.assert_called_once_with(
            '18d8be353ed3480476f032475e7c233eff7371d5')

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_raw_ok_no_msg(self, mock_service):
        # given
        mock_service.lookup_revision_message.side_effect = NotFoundExc(
            'No message for revision')

        # when
        rv = self.client.get('/api/1/revision/'
                             '18d8be353ed3480476f032475e7c233eff7371d5/raw/')

        # then
        self.assertEqual(rv.status_code, 404)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, {
            'exception': 'NotFoundExc',
            'reason': 'No message for revision'})

        self.assertEqual
        mock_service.lookup_revision_message.assert_called_once_with(
            '18d8be353ed3480476f032475e7c233eff7371d5')

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_raw_ko_no_rev(self, mock_service):
        # given
        mock_service.lookup_revision_message.side_effect = NotFoundExc(
            'No revision found')

        # when
        rv = self.client.get('/api/1/revision/'
                             '18d8be353ed3480476f032475e7c233eff7371d5/raw/')

        # then
        self.assertEqual(rv.status_code, 404)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, {
            'exception': 'NotFoundExc',
            'reason': 'No revision found'})

        mock_service.lookup_revision_message.assert_called_once_with(
            '18d8be353ed3480476f032475e7c233eff7371d5')

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_with_origin_not_found(self, mock_service):
        mock_service.lookup_revision_by.return_value = None

        rv = self.client.get('/api/1/revision/origin/123/')

        # then
        self.assertEqual(rv.status_code, 404)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertIn('Revision with (origin_id: 123', rv.data['reason'])
        self.assertIn('not found', rv.data['reason'])
        self.assertEqual('NotFoundExc', rv.data['exception'])

        mock_service.lookup_revision_by.assert_called_once_with(
            '123',
            'HEAD',
            None)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_with_origin(self, mock_service):
        mock_revision = {
            'id': '32',
            'directory': '21',
            'message': 'message 1',
            'type': 'deb',
        }
        expected_revision = {
            'id': '32',
            'url': '/api/1/revision/32/',
            'history_url': '/api/1/revision/32/log/',
            'directory': '21',
            'directory_url': '/api/1/directory/21/',
            'message': 'message 1',
            'type': 'deb',
        }
        mock_service.lookup_revision_by.return_value = mock_revision

        rv = self.client.get('/api/1/revision/origin/1/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, expected_revision)

        mock_service.lookup_revision_by.assert_called_once_with(
            '1',
            'HEAD',
            None)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_with_origin_and_branch_name(self, mock_service):
        mock_revision = {
            'id': '12',
            'directory': '23',
            'message': 'message 2',
            'type': 'tar',
        }
        mock_service.lookup_revision_by.return_value = mock_revision

        expected_revision = {
            'id': '12',
            'url': '/api/1/revision/12/',
            'history_url': '/api/1/revision/12/log/',
            'directory': '23',
            'directory_url': '/api/1/directory/23/',
            'message': 'message 2',
            'type': 'tar',
        }

        rv = self.client.get('/api/1/revision/origin/1'
                             '/branch/refs/origin/dev/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, expected_revision)

        mock_service.lookup_revision_by.assert_called_once_with(
            '1',
            'refs/origin/dev',
            None)

    @patch('swh.web.api.views.revision.service')
    @patch('swh.web.api.views.revision.utils')
    def test_api_revision_with_origin_and_branch_name_and_timestamp(self,
                                                               mock_utils,
                                                               mock_service): # noqa
        mock_revision = {
            'id': '123',
            'directory': '456',
            'message': 'message 3',
            'type': 'tar',
        }
        mock_service.lookup_revision_by.return_value = mock_revision

        expected_revision = {
            'id': '123',
            'url': '/api/1/revision/123/',
            'history_url': '/api/1/revision/123/log/',
            'directory': '456',
            'directory_url': '/api/1/directory/456/',
            'message': 'message 3',
            'type': 'tar',
        }

        mock_utils.enrich_revision.return_value = expected_revision

        rv = self.client.get('/api/1/revision'
                             '/origin/1'
                             '/branch/refs/origin/dev'
                             '/ts/1452591542/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, expected_revision)

        mock_service.lookup_revision_by.assert_called_once_with(
            '1',
            'refs/origin/dev',
            '1452591542')
        mock_utils.enrich_revision.assert_called_once_with(
            mock_revision)

    @patch('swh.web.api.views.revision.service')
    @patch('swh.web.api.views.revision.utils')
    def test_api_revision_with_origin_and_branch_name_and_timestamp_escapes(
            self,
            mock_utils,
            mock_service):
        mock_revision = {
            'id': '999',
        }
        mock_service.lookup_revision_by.return_value = mock_revision

        expected_revision = {
            'id': '999',
            'url': '/api/1/revision/999/',
            'history_url': '/api/1/revision/999/log/',
        }

        mock_utils.enrich_revision.return_value = expected_revision

        rv = self.client.get('/api/1/revision'
                             '/origin/1'
                             '/branch/refs%2Forigin%2Fdev'
                             '/ts/Today%20is%20'
                             'January%201,%202047%20at%208:21:00AM/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, expected_revision)

        mock_service.lookup_revision_by.assert_called_once_with(
            '1',
            'refs/origin/dev',
            'Today is January 1, 2047 at 8:21:00AM')
        mock_utils.enrich_revision.assert_called_once_with(
            mock_revision)

    @patch('swh.web.api.views.revision.service')
    def test_revision_directory_by_ko_raise(self, mock_service):
        # given
        mock_service.lookup_directory_through_revision.side_effect = NotFoundExc('not')  # noqa

        # when
        with self.assertRaises(NotFoundExc):
            _revision_directory_by(
                {'sha1_git': 'id'},
                None,
                '/api/1/revision/sha1/directory/')

        # then
        mock_service.lookup_directory_through_revision.assert_called_once_with(
            {'sha1_git': 'id'},
            None, limit=100, with_data=False)

    @patch('swh.web.api.views.revision.service')
    def test_revision_directory_by_type_dir(self, mock_service):
        # given
        mock_service.lookup_directory_through_revision.return_value = (
            'rev-id',
            {
                'type': 'dir',
                'revision': 'rev-id',
                'path': 'some/path',
                'content': []
            })
        # when
        actual_dir_content = _revision_directory_by(
            {'sha1_git': 'blah-id'},
            'some/path', '/api/1/revision/sha1/directory/')

        # then
        self.assertEqual(actual_dir_content, {
            'type': 'dir',
            'revision': 'rev-id',
            'path': 'some/path',
            'content': []
        })

        mock_service.lookup_directory_through_revision.assert_called_once_with(
            {'sha1_git': 'blah-id'},
            'some/path', limit=100, with_data=False)

    @patch('swh.web.api.views.revision.service')
    def test_revision_directory_by_type_file(self, mock_service):
        # given
        mock_service.lookup_directory_through_revision.return_value = (
            'rev-id',
            {
                'type': 'file',
                'revision': 'rev-id',
                'path': 'some/path',
                'content': {'blah': 'blah'}
            })
        # when
        actual_dir_content = _revision_directory_by(
            {'sha1_git': 'sha1'},
            'some/path',
            '/api/1/revision/origin/2/directory/',
            limit=1000, with_data=True)

        # then
        self.assertEqual(actual_dir_content, {
                'type': 'file',
                'revision': 'rev-id',
                'path': 'some/path',
                'content': {'blah': 'blah'}
            })

        mock_service.lookup_directory_through_revision.assert_called_once_with(
            {'sha1_git': 'sha1'},
            'some/path', limit=1000, with_data=True)

    @patch('swh.web.api.views.revision.parse_timestamp')
    @patch('swh.web.api.views.revision._revision_directory_by')
    @patch('swh.web.api.views.revision.utils')
    def test_api_directory_through_revision_origin_ko_not_found(self,
                                                           mock_utils,
                                                           mock_rev_dir,
                                                           mock_parse_timestamp): # noqa
        mock_rev_dir.side_effect = NotFoundExc('not found')
        mock_parse_timestamp.return_value = '2012-10-20 00:00:00'

        rv = self.client.get('/api/1/revision'
                             '/origin/10'
                             '/branch/refs/remote/origin/dev'
                             '/ts/2012-10-20'
                             '/directory/')

        # then
        self.assertEqual(rv.status_code, 404)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, {
            'exception': 'NotFoundExc',
            'reason': 'not found'})

        mock_rev_dir.assert_called_once_with(
            {'origin_id': '10',
             'branch_name': 'refs/remote/origin/dev',
             'ts': '2012-10-20 00:00:00'}, None,
            '/api/1/revision'
            '/origin/10'
            '/branch/refs/remote/origin/dev'
            '/ts/2012-10-20'
            '/directory/',
            with_data=False)

    @patch('swh.web.api.views.revision._revision_directory_by')
    def test_api_directory_through_revision_origin(self,
                                                   mock_revision_dir):
        expected_res = [{
            'id': '123'
        }]
        mock_revision_dir.return_value = expected_res

        rv = self.client.get('/api/1/revision/origin/3/directory/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, expected_res)

        mock_revision_dir.assert_called_once_with({
            'origin_id': '3',
            'branch_name': 'refs/heads/master',
            'ts': None}, None, '/api/1/revision/origin/3/directory/',
                                                  with_data=False)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_log(self, mock_service):
        # given
        stub_revisions = [{
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'author_name': 'Software Heritage',
            'author_email': 'robot@softwareheritage.org',
            'committer_name': 'Software Heritage',
            'committer_email': 'robot@softwareheritage.org',
            'message': 'synthetic revision message',
            'date_offset': 0,
            'committer_date_offset': 0,
            'parents': ['7834ef7e7c357ce2af928115c6c6a42b7e2a4345'],
            'type': 'tar',
            'synthetic': True,
        }]
        mock_service.lookup_revision_log.return_value = stub_revisions

        expected_revisions = [{
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'url': '/api/1/revision/18d8be353ed3480476f032475e7c233eff7371d5/',
            'history_url': '/api/1/revision/18d8be353ed3480476f032475e7c233ef'
            'f7371d5/log/',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'directory_url': '/api/1/directory/7834ef7e7c357ce2af928115c6c6a'
            '42b7e2a44e6/',
            'author_name': 'Software Heritage',
            'author_email': 'robot@softwareheritage.org',
            'committer_name': 'Software Heritage',
            'committer_email': 'robot@softwareheritage.org',
            'message': 'synthetic revision message',
            'date_offset': 0,
            'committer_date_offset': 0,
            'parents': [{
                'id': '7834ef7e7c357ce2af928115c6c6a42b7e2a4345',
                'url': '/api/1/revision/7834ef7e7c357ce2af928115c6c6a42b7e2a4345/',  # noqa
            }],
            'type': 'tar',
            'synthetic': True,
        }]

        # when
        rv = self.client.get('/api/1/revision/8834ef7e7c357ce2af928115c6c6a42'
                             'b7e2a44e6/log/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')

        self.assertEqual(rv.data, expected_revisions)
        self.assertFalse(rv.has_header('Link'))

        mock_service.lookup_revision_log.assert_called_once_with(
            '8834ef7e7c357ce2af928115c6c6a42b7e2a44e6', 11)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_log_with_next(self, mock_service):
        # given
        stub_revisions = []
        for i in range(27):
            stub_revisions.append({'id': str(i)})

        mock_service.lookup_revision_log.return_value = stub_revisions[:26]

        expected_revisions = [x for x in stub_revisions if int(x['id']) < 25]
        for e in expected_revisions:
            e['url'] = '/api/1/revision/%s/' % e['id']
            e['history_url'] = '/api/1/revision/%s/log/' % e['id']

        # when
        rv = self.client.get('/api/1/revision/8834ef7e7c357ce2af928115c6c6a42'
                             'b7e2a44e6/log/?per_page=25')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, expected_revisions)
        self.assertEqual(rv['Link'],
                         '</api/1/revision/25/log/?per_page=25>; rel="next"')

        mock_service.lookup_revision_log.assert_called_once_with(
            '8834ef7e7c357ce2af928115c6c6a42b7e2a44e6', 26)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_log_not_found(self, mock_service):
        # given
        mock_service.lookup_revision_log.return_value = None

        # when
        rv = self.client.get('/api/1/revision/8834ef7e7c357ce2af928115c6c6'
                             'a42b7e2a44e6/log/')

        # then
        self.assertEqual(rv.status_code, 404)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, {
            'exception': 'NotFoundExc',
            'reason': 'Revision with sha1_git'
            ' 8834ef7e7c357ce2af928115c6c6a42b7e2a44e6 not found.'})
        self.assertFalse(rv.has_header('Link'))

        mock_service.lookup_revision_log.assert_called_once_with(
            '8834ef7e7c357ce2af928115c6c6a42b7e2a44e6', 11)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_log_context(self, mock_service):
        # given
        stub_revisions = [{
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'author_name': 'Software Heritage',
            'author_email': 'robot@softwareheritage.org',
            'committer_name': 'Software Heritage',
            'committer_email': 'robot@softwareheritage.org',
            'message': 'synthetic revision message',
            'date_offset': 0,
            'committer_date_offset': 0,
            'parents': ['7834ef7e7c357ce2af928115c6c6a42b7e2a4345'],
            'type': 'tar',
            'synthetic': True,
        }]

        mock_service.lookup_revision_log.return_value = stub_revisions
        mock_service.lookup_revision_multiple.return_value = [{
            'id': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'directory': '18d8be353ed3480476f032475e7c233eff7371d5',
            'author_name': 'Name Surname',
            'author_email': 'name@surname.com',
            'committer_name': 'Name Surname',
            'committer_email': 'name@surname.com',
            'message': 'amazing revision message',
            'date_offset': 0,
            'committer_date_offset': 0,
            'parents': ['adc83b19e793491b1c6ea0fd8b46cd9f32e592fc'],
            'type': 'tar',
            'synthetic': True,
        }]

        expected_revisions = [
            {
                'url': '/api/1/revision/'
                '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6/',
                'history_url': '/api/1/revision/'
                '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6/log/',
                'id': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
                'directory': '18d8be353ed3480476f032475e7c233eff7371d5',
                'directory_url': '/api/1/directory/'
                '18d8be353ed3480476f032475e7c233eff7371d5/',
                'author_name': 'Name Surname',
                'author_email': 'name@surname.com',
                'committer_name': 'Name Surname',
                'committer_email': 'name@surname.com',
                'message': 'amazing revision message',
                'date_offset': 0,
                'committer_date_offset': 0,
                'parents': [{
                    'id': 'adc83b19e793491b1c6ea0fd8b46cd9f32e592fc',
                    'url': '/api/1/revision/adc83b19e793491b1c6ea0fd8b46cd9f32e592fc/',  # noqa
                }],
                'type': 'tar',
                'synthetic': True,
            },
            {
                'url': '/api/1/revision/'
                '18d8be353ed3480476f032475e7c233eff7371d5/',
                'history_url': '/api/1/revision/'
                '18d8be353ed3480476f032475e7c233eff7371d5/log/',
                'id': '18d8be353ed3480476f032475e7c233eff7371d5',
                'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
                'directory_url': '/api/1/directory/'
                '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6/',
                'author_name': 'Software Heritage',
                'author_email': 'robot@softwareheritage.org',
                'committer_name': 'Software Heritage',
                'committer_email': 'robot@softwareheritage.org',
                'message': 'synthetic revision message',
                'date_offset': 0,
                'committer_date_offset': 0,
                'parents': [{
                    'id': '7834ef7e7c357ce2af928115c6c6a42b7e2a4345',
                    'url': '/api/1/revision/7834ef7e7c357ce2af928115c6c6a42b7e2a4345/',  # noqa
                }],
                'type': 'tar',
                'synthetic': True,
            }]

        # when
        rv = self.client.get('/api/1/revision/18d8be353ed3480476f0'
                             '32475e7c233eff7371d5/prev/21145781e2'
                             '6ad1f978e/log/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(expected_revisions, rv.data)
        self.assertFalse(rv.has_header('Link'))

        mock_service.lookup_revision_log.assert_called_once_with(
            '18d8be353ed3480476f032475e7c233eff7371d5', 11)
        mock_service.lookup_revision_multiple.assert_called_once_with(
            ['21145781e26ad1f978e'])

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_log_by(self, mock_service):
        # given
        stub_revisions = [{
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'author_name': 'Software Heritage',
            'author_email': 'robot@softwareheritage.org',
            'committer_name': 'Software Heritage',
            'committer_email': 'robot@softwareheritage.org',
            'message': 'synthetic revision message',
            'date_offset': 0,
            'committer_date_offset': 0,
            'parents': ['7834ef7e7c357ce2af928115c6c6a42b7e2a4345'],
            'type': 'tar',
            'synthetic': True,
        }]
        mock_service.lookup_revision_log_by.return_value = stub_revisions

        expected_revisions = [{
            'id': '18d8be353ed3480476f032475e7c233eff7371d5',
            'url': '/api/1/revision/18d8be353ed3480476f032475e7c233eff7371d5/',
            'history_url': '/api/1/revision/18d8be353ed3480476f032475e7c233ef'
                           'f7371d5/log/',
            'directory': '7834ef7e7c357ce2af928115c6c6a42b7e2a44e6',
            'directory_url': '/api/1/directory/7834ef7e7c357ce2af928115c6c6a'
                             '42b7e2a44e6/',
            'author_name': 'Software Heritage',
            'author_email': 'robot@softwareheritage.org',
            'committer_name': 'Software Heritage',
            'committer_email': 'robot@softwareheritage.org',
            'message': 'synthetic revision message',
            'date_offset': 0,
            'committer_date_offset': 0,
            'parents': [{
                'id': '7834ef7e7c357ce2af928115c6c6a42b7e2a4345',
                 'url': '/api/1/revision/7834ef7e7c357ce2af928115c6c6a42b7e2a4345/'  # noqa
            }],
            'type': 'tar',
            'synthetic': True,
        }]

        # when
        rv = self.client.get('/api/1/revision/origin/1/log/')

        # then
        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, expected_revisions)
        self.assertFalse(rv.has_header('Link'))

        mock_service.lookup_revision_log_by.assert_called_once_with(
            '1', 'HEAD', None, 11)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_log_by_with_next(self, mock_service):
        # given
        stub_revisions = []
        for i in range(27):
            stub_revisions.append({'id': str(i)})

        mock_service.lookup_revision_log_by.return_value = stub_revisions[:26]

        expected_revisions = [x for x in stub_revisions if int(x['id']) < 25]
        for e in expected_revisions:
            e['url'] = '/api/1/revision/%s/' % e['id']
            e['history_url'] = '/api/1/revision/%s/log/' % e['id']

        # when
        rv = self.client.get('/api/1/revision/origin/1/log/?per_page=25')

        # then
        self.assertEqual(rv.status_code, 200)

        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertIsNotNone(rv['Link'])
        self.assertEqual(rv.data, expected_revisions)

        mock_service.lookup_revision_log_by.assert_called_once_with(
            '1', 'HEAD', None, 26)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_log_by_norev(self, mock_service):
        # given
        mock_service.lookup_revision_log_by.side_effect = NotFoundExc(
            'No revision')

        # when
        rv = self.client.get('/api/1/revision/origin/1/log/')

        # then
        self.assertEqual(rv.status_code, 404)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertFalse(rv.has_header('Link'))
        self.assertEqual(rv.data, {'exception': 'NotFoundExc',
                                   'reason': 'No revision'})

        mock_service.lookup_revision_log_by.assert_called_once_with(
            '1', 'HEAD', None, 11)

    @patch('swh.web.api.views.revision.service')
    def test_api_revision_history(self, mock_service):
        # for readability purposes, we use:
        # - sha1 as 3 letters (url are way too long otherwise to respect pep8)
        # - only keys with modification steps (all other keys are kept as is)

        # given
        stub_revision = {
            'id': '883',
            'children': ['777', '999'],
            'parents': [],
            'directory': '272'
        }

        mock_service.lookup_revision.return_value = stub_revision

        # then
        rv = self.client.get('/api/1/revision/883/prev/999/')

        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, {
            'id': '883',
            'url': '/api/1/revision/883/',
            'history_url': '/api/1/revision/883/log/',
            'history_context_url': '/api/1/revision/883/prev/999/log/',
            'children': ['777', '999'],
            'children_urls': ['/api/1/revision/777/',
                              '/api/1/revision/999/'],
            'parents': [],
            'directory': '272',
            'directory_url': '/api/1/directory/272/'
        })

        mock_service.lookup_revision.assert_called_once_with('883')

    @patch('swh.web.api.views.revision._revision_directory_by')
    def test_api_revision_directory_ko_not_found(self, mock_rev_dir):
        # given
        mock_rev_dir.side_effect = NotFoundExc('Not found')

        # then
        rv = self.client.get('/api/1/revision/999/directory/some/path/to/dir/')

        self.assertEqual(rv.status_code, 404)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, {
            'exception': 'NotFoundExc',
            'reason': 'Not found'})

        mock_rev_dir.assert_called_once_with(
            {'sha1_git': '999'},
            'some/path/to/dir',
            '/api/1/revision/999/directory/some/path/to/dir/',
            with_data=False)

    @patch('swh.web.api.views.revision._revision_directory_by')
    def test_api_revision_directory_ok_returns_dir_entries(self, mock_rev_dir):
        stub_dir = {
            'type': 'dir',
            'revision': '999',
            'content': [
                {
                    'sha1_git': '789',
                    'type': 'file',
                    'target': '101',
                    'target_url': '/api/1/content/sha1_git:101/',
                    'name': 'somefile',
                    'file_url': '/api/1/revision/999/directory/some/path/'
                    'somefile/'
                },
                {
                    'sha1_git': '123',
                    'type': 'dir',
                    'target': '456',
                    'target_url': '/api/1/directory/456/',
                    'name': 'to-subdir',
                    'dir_url': '/api/1/revision/999/directory/some/path/'
                    'to-subdir/',
                }]
        }

        # given
        mock_rev_dir.return_value = stub_dir

        # then
        rv = self.client.get('/api/1/revision/999/directory/some/path/')

        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, stub_dir)

        mock_rev_dir.assert_called_once_with(
            {'sha1_git': '999'},
            'some/path',
            '/api/1/revision/999/directory/some/path/',
            with_data=False)

    @patch('swh.web.api.views.revision._revision_directory_by')
    def test_api_revision_directory_ok_returns_content(self, mock_rev_dir):
        stub_content = {
            'type': 'file',
            'revision': '999',
            'content': {
                'sha1_git': '789',
                'sha1': '101',
                'data_url': '/api/1/content/101/raw/',
            }
        }

        # given
        mock_rev_dir.return_value = stub_content

        # then
        url = '/api/1/revision/666/directory/some/other/path/'
        rv = self.client.get(url)

        self.assertEqual(rv.status_code, 200)
        self.assertEqual(rv['Content-Type'], 'application/json')
        self.assertEqual(rv.data, stub_content)

        mock_rev_dir.assert_called_once_with(
            {'sha1_git': '666'}, 'some/other/path', url, with_data=False)



# Copyright (C) 2015-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

import functools

from django.http import HttpResponse

from swh.web.common import service
from swh.web.common.utils import reverse
from swh.web.common.exc import NotFoundExc
from swh.web.api.apidoc import api_doc
from swh.web.api import utils
from swh.web.api.apiurls import api_route
from swh.web.api.views.utils import api_lookup


@api_route(r'/content/(?P<q>.+)/filetype/', 'api-content-filetype')
@api_doc('/content/filetype/')
def api_content_filetype(request, q):
    """
    .. http:get:: /api/1/content/[(hash_type):](hash)/filetype/

        Get information about the detected MIME type of a content object.

        :param string hash_type: optional parameter specifying which hashing algorithm has been used
            to compute the content checksum. It can be either ``sha1``, ``sha1_git``, ``sha256``
            or ``blake2s256``. If that parameter is not provided, it is assumed that the
            hashing algorithm used is `sha1`.
        :param string hash: hexadecimal representation of the checksum value computed with
            the specified hashing algorithm.

        :>json object content_url: link to :http:get:`/api/1/content/[(hash_type):](hash)/` for
            getting information about the content
        :>json string encoding: the detected content encoding
        :>json string id: the **sha1** identifier of the content
        :>json string mimetype: the detected MIME type of the content
        :>json object tool: information about the tool used to detect the content filetype

        :reqheader Accept: the requested response content type,
            either ``application/json`` (default) or ``application/yaml``
        :resheader Content-Type: this depends on :http:header:`Accept` header of request

        **Allowed HTTP Methods:** :http:method:`get`, :http:method:`head`, :http:method:`options`

        :statuscode 200: no error
        :statuscode 400: an invalid **hash_type** or **hash** has been provided
        :statuscode 404: requested content can not be found in the archive

        **Example:**

        .. parsed-literal::

            :swh_web_api:`content/sha1:dc2830a9e72f23c1dfebef4413003221baa5fb62/filetype/`
    """ # noqa
    return api_lookup(
        service.lookup_content_filetype, q,
        notfound_msg='No filetype information found for content {}.'.format(q),
        enrich_fn=utils.enrich_metadata_endpoint)


@api_route(r'/content/(?P<q>.+)/language/', 'api-content-language')
@api_doc('/content/language/')
def api_content_language(request, q):
    """
    .. http:get:: /api/1/content/[(hash_type):](hash)/language/

        Get information about the programming language used in a content object.

        :param string hash_type: optional parameter specifying which hashing algorithm has been used
            to compute the content checksum. It can be either ``sha1``, ``sha1_git``, ``sha256``
            or ``blake2s256``. If that parameter is not provided, it is assumed that the
            hashing algorithm used is ``sha1``.
        :param string hash: hexadecimal representation of the checksum value computed with
            the specified hashing algorithm.

        :>json object content_url: link to :http:get:`/api/1/content/[(hash_type):](hash)/` for
            getting information about the content
        :>json string id: the **sha1** identifier of the content
        :>json string lang: the detected programming language if any
        :>json object tool: information about the tool used to detect the programming language

        :reqheader Accept: the requested response content type,
            either ``application/json`` (default) or ``application/yaml``
        :resheader Content-Type: this depends on :http:header:`Accept` header of request

        **Allowed HTTP Methods:** :http:method:`get`, :http:method:`head`, :http:method:`options`

        :statuscode 200: no error
        :statuscode 400: an invalid **hash_type** or **hash** has been provided
        :statuscode 404: requested content can not be found in the archive

        **Example:**

        .. parsed-literal::

            :swh_web_api:`content/sha1:dc2830a9e72f23c1dfebef4413003221baa5fb62/language/`
    """ # noqa
    return api_lookup(
        service.lookup_content_language, q,
        notfound_msg='No language information found for content {}.'.format(q),
        enrich_fn=utils.enrich_metadata_endpoint)


@api_route(r'/content/(?P<q>.+)/license/', 'api-content-license')
@api_doc('/content/license/')
def api_content_license(request, q):
    """
    .. http:get:: /api/1/content/[(hash_type):](hash)/license/

        Get information about the license of a content object.

        :param string hash_type: optional parameter specifying which hashing algorithm has been used
            to compute the content checksum. It can be either ``sha1``, ``sha1_git``, ``sha256``
            or ``blake2s256``. If that parameter is not provided, it is assumed that the
            hashing algorithm used is ``sha1``.
        :param string hash: hexadecimal representation of the checksum value computed with
            the specified hashing algorithm.

        :>json object content_url: link to :http:get:`/api/1/content/[(hash_type):](hash)/` for
            getting information about the content
        :>json string id: the **sha1** identifier of the content
        :>json array licenses: array of strings containing the detected license names if any
        :>json object tool: information about the tool used to detect the license

        :reqheader Accept: the requested response content type,
            either ``application/json`` (default) or ``application/yaml``
        :resheader Content-Type: this depends on :http:header:`Accept` header of request

        **Allowed HTTP Methods:** :http:method:`get`, :http:method:`head`, :http:method:`options`

        :statuscode 200: no error
        :statuscode 400: an invalid **hash_type** or **hash** has been provided
        :statuscode 404: requested content can not be found in the archive

        **Example:**

        .. parsed-literal::

            :swh_web_api:`content/sha1:dc2830a9e72f23c1dfebef4413003221baa5fb62/license/`
    """ # noqa
    return api_lookup(
        service.lookup_content_license, q,
        notfound_msg='No license information found for content {}.'.format(q),
        enrich_fn=utils.enrich_metadata_endpoint)


@api_route(r'/content/(?P<q>.+)/ctags/', 'api-content-ctags')
@api_doc('/content/ctags/', tags=['hidden'])
def api_content_ctags(request, q):
    """
    Get information about all `Ctags <http://ctags.sourceforge.net/>`_-style
    symbols defined in a content object.
    """
    return api_lookup(
        service.lookup_content_ctags, q,
        notfound_msg='No ctags symbol found for content {}.'.format(q),
        enrich_fn=utils.enrich_metadata_endpoint)


@api_route(r'/content/(?P<q>.+)/raw/', 'api-content-raw')
@api_doc('/content/raw/', handle_response=True)
def api_content_raw(request, q):
    """
    .. http:get:: /api/1/content/[(hash_type):](hash)/raw/

        Get the raw content of a content object (aka a "blob"), as a byte sequence.

        :param string hash_type: optional parameter specifying which hashing algorithm has been used
            to compute the content checksum. It can be either ``sha1``, ``sha1_git``, ``sha256``
            or ``blake2s256``. If that parameter is not provided, it is assumed that the
            hashing algorithm used is ``sha1``.
        :param string hash: hexadecimal representation of the checksum value computed with
            the specified hashing algorithm.
        :query string filename: if provided, the downloaded content will get that filename

        :resheader Content-Type: application/octet-stream

        **Allowed HTTP Methods:** :http:method:`get`, :http:method:`head`, :http:method:`options`

        :statuscode 200: no error
        :statuscode 400: an invalid **hash_type** or **hash** has been provided
        :statuscode 404: requested content can not be found in the archive

        **Example:**

        .. parsed-literal::

            :swh_web_api:`content/sha1:dc2830a9e72f23c1dfebef4413003221baa5fb62/raw/`
    """ # noqa
    def generate(content):
        yield content['data']

    content_raw = service.lookup_content_raw(q)
    if not content_raw:
        raise NotFoundExc('Content %s is not found.' % q)

    filename = request.query_params.get('filename')
    if not filename:
        filename = 'content_%s_raw' % q.replace(':', '_')

    response = HttpResponse(generate(content_raw),
                            content_type='application/octet-stream')
    response['Content-disposition'] = 'attachment; filename=%s' % filename
    return response


@api_route(r'/content/symbol/(?P<q>.+)/', 'api-content-symbol')
@api_doc('/content/symbol/', tags=['hidden'])
def api_content_symbol(request, q=None):
    """Search content objects by `Ctags <http://ctags.sourceforge.net/>`_-style
    symbol (e.g., function name, data type, method, ...).

    """
    result = {}
    last_sha1 = request.query_params.get('last_sha1', None)
    per_page = int(request.query_params.get('per_page', '10'))

    def lookup_exp(exp, last_sha1=last_sha1, per_page=per_page):
        return service.lookup_expression(exp, last_sha1, per_page)

    symbols = api_lookup(
        lookup_exp, q,
        notfound_msg="No indexed raw content match expression '{}'.".format(q),
        enrich_fn=functools.partial(utils.enrich_content, top_url=True))

    if symbols:
        nb_symbols = len(symbols)

        if nb_symbols == per_page:
            query_params = {}
            new_last_sha1 = symbols[-1]['sha1']
            query_params['last_sha1'] = new_last_sha1
            if request.query_params.get('per_page'):
                query_params['per_page'] = per_page

            result['headers'] = {
                'link-next': reverse('api-content-symbol', url_args={'q': q},
                                     query_params=query_params)
            }

    result.update({
        'results': symbols
    })

    return result


@api_route(r'/content/known/search/', 'api-content-known', methods=['POST'])
@api_route(r'/content/known/(?P<q>(?!search).*)/', 'api-content-known')
@api_doc('/content/known/', tags=['hidden'])
def api_check_content_known(request, q=None):
    """
    .. http:get:: /api/1/content/known/(sha1)[,(sha1), ...,(sha1)]/

        Check whether some content(s) (aka "blob(s)") is present in the archive
        based on its **sha1** checksum.

        :param string sha1: hexadecimal representation of the **sha1** checksum value
            for the content to check existence. Multiple values can be provided separated
            by ','.

        :reqheader Accept: the requested response content type,
            either ``application/json`` (default) or ``application/yaml``
        :resheader Content-Type: this depends on :http:header:`Accept` header of request

        :>json array search_res: array holding the search result for each provided **sha1**
        :>json object search_stats: some statistics regarding the number of **sha1** provided
            and the percentage of those found in the archive

        **Allowed HTTP Methods:** :http:method:`get`, :http:method:`head`, :http:method:`options`

        :statuscode 200: no error
        :statuscode 400: an invalid **sha1** has been provided

        **Example:**

        .. parsed-literal::

            :swh_web_api:`content/known/dc2830a9e72f23c1dfebef4413003221baa5fb62,0c3f19cb47ebfbe643fb19fa94c874d18fa62d12/`
    """ # noqa
    response = {'search_res': None,
                'search_stats': None}
    search_stats = {'nbfiles': 0, 'pct': 0}
    search_res = None

    queries = []
    # GET: Many hash separated values request
    if q:
        hashes = q.split(',')
        for v in hashes:
            queries.append({'filename': None, 'sha1': v})

    # POST: Many hash requests in post form submission
    elif request.method == 'POST':
        data = request.data
        # Remove potential inputs with no associated value
        for k, v in data.items():
            if v is not None:
                if k == 'q' and len(v) > 0:
                    queries.append({'filename': None, 'sha1': v})
                elif v != '':
                    queries.append({'filename': k, 'sha1': v})

    if queries:
        lookup = service.lookup_multiple_hashes(queries)
        result = []
        nb_queries = len(queries)
        for el in lookup:
            res_d = {'sha1': el['sha1'],
                     'found': el['found']}
            if 'filename' in el and el['filename']:
                res_d['filename'] = el['filename']
            result.append(res_d)
            search_res = result
            nbfound = len([x for x in lookup if x['found']])
            search_stats['nbfiles'] = nb_queries
            search_stats['pct'] = (nbfound / nb_queries) * 100

    response['search_res'] = search_res
    response['search_stats'] = search_stats
    return response


@api_route(r'/content/(?P<q>.+)/', 'api-content')
@api_doc('/content/')
def api_content_metadata(request, q):
    """
    .. http:get:: /api/1/content/[(hash_type):](hash)/

        Get information about a content (aka a "blob") object.
        In the archive, a content object is identified based on checksum
        values computed using various hashing algorithms.

        :param string hash_type: optional parameter specifying which hashing algorithm has been used
            to compute the content checksum. It can be either ``sha1``, ``sha1_git``, ``sha256``
            or ``blake2s256``. If that parameter is not provided, it is assumed that the
            hashing algorithm used is ``sha1``.
        :param string hash: hexadecimal representation of the checksum value computed with
            the specified hashing algorithm.

        :reqheader Accept: the requested response content type,
            either ``application/json`` (default) or ``application/yaml``
        :resheader Content-Type: this depends on :http:header:`Accept` header of request

        :>json object checksums: object holding the computed checksum values for the requested content
        :>json string data_url: link to :http:get:`/api/1/content/[(hash_type):](hash)/raw/`
            for downloading the content raw bytes
        :>json string filetype_url: link to :http:get:`/api/1/content/[(hash_type):](hash)/filetype/`
            for getting information about the content MIME type
        :>json string language_url: link to :http:get:`/api/1/content/[(hash_type):](hash)/language/`
            for getting information about the programming language used in the content
        :>json number length: length of the content in bytes
        :>json string license_url: link to :http:get:`/api/1/content/[(hash_type):](hash)/license/`
            for getting information about the license of the content

        **Allowed HTTP Methods:** :http:method:`get`, :http:method:`head`, :http:method:`options`

        :statuscode 200: no error
        :statuscode 400: an invalid **hash_type** or **hash** has been provided
        :statuscode 404: requested content can not be found in the archive

        **Example:**

        .. parsed-literal::

            curl -i :swh_web_api:`content/sha1_git:fe95a46679d128ff167b7c55df5d02356c5a1ae1/`
    """ # noqa
    return api_lookup(
        service.lookup_content, q,
        notfound_msg='Content with {} not found.'.format(q),
        enrich_fn=functools.partial(utils.enrich_content, query_string=q))



# Copyright (C) 2015-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from rest_framework.response import Response
from rest_framework.decorators import api_view

from types import GeneratorType

from swh.web.common.exc import NotFoundExc
from swh.web.api.apiurls import APIUrls, api_route


def api_lookup(lookup_fn, *args,
               notfound_msg='Object not found',
               enrich_fn=None):
    """
    Capture a redundant behavior of:
        - looking up the backend with a criteria (be it an identifier or
          checksum) passed to the function lookup_fn
        - if nothing is found, raise an NotFoundExc exception with error
          message notfound_msg.
        - Otherwise if something is returned:
            - either as list, map or generator, map the enrich_fn function to
              it and return the resulting data structure as list.
            - either as dict and pass to enrich_fn and return the dict
              enriched.

    Args:
        - lookup_fn: function expects one criteria and optional supplementary
          \*args.
        - notfound_msg: if nothing matching the criteria is found,
          raise NotFoundExc with this error message.
        - enrich_fn: Function to use to enrich the result returned by
          lookup_fn. Default to the identity function if not provided.
        - \*args: supplementary arguments to pass to lookup_fn.

    Raises:
        NotFoundExp or whatever `lookup_fn` raises.

    """ # noqa
    if enrich_fn is None:
        enrich_fn = (lambda x: x)
    res = lookup_fn(*args)
    if not res:
        raise NotFoundExc(notfound_msg)
    if isinstance(res, (map, list, GeneratorType)):
        return [enrich_fn(x) for x in res]
    return enrich_fn(res)


@api_view(['GET', 'HEAD'])
def api_home(request):
    return Response({}, template_name='api/api.html')


APIUrls.add_url_pattern(r'^$', api_home, view_name='api-homepage')


@api_route(r'/', 'api-endpoints')
def api_endpoints(request):
    """Display the list of opened api endpoints.

    """
    routes = APIUrls.get_app_endpoints().copy()
    for route, doc in routes.items():
        doc['doc_intro'] = doc['docstring'].split('\n\n')[0]
    # Return a list of routes with consistent ordering
    env = {
        'doc_routes': sorted(routes.items())
    }
    return Response(env, template_name="api/endpoints.html")



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

# flake8: noqa

from unittest.mock import patch, MagicMock
from django.utils.html import escape

from swh.web.common.exc import NotFoundExc
from swh.web.common.utils import (
    reverse, format_utc_iso_date, get_swh_persistent_id,
    parse_timestamp
)
from swh.web.tests.testcase import WebTestCase

from .data.revision_test_data import (
    revision_id_test, revision_metadata_test,
    revision_history_log_test
)

from .data.origin_test_data import stub_origin_visits, stub_origin_snapshot


class SwhBrowseRevisionTest(WebTestCase):

    @patch('swh.web.browse.utils.get_origin_visit_snapshot')
    @patch('swh.web.browse.views.revision.service')
    @patch('swh.web.browse.utils.service')
    @patch('swh.web.common.origin_visits.get_origin_visits')
    def test_revision_browse(self, mock_get_origin_visits, mock_service_utils,
                             mock_service, mock_get_origin_visit_snapshot):
        mock_service.lookup_revision.return_value = revision_metadata_test

        url = reverse('browse-revision',
                      url_args={'sha1_git': revision_id_test})

        author_id = revision_metadata_test['author']['id']
        author_name = revision_metadata_test['author']['name']
        committer_id = revision_metadata_test['committer']['id']
        committer_name = revision_metadata_test['committer']['name']
        dir_id = revision_metadata_test['directory']

        author_url = reverse('browse-person',
                             url_args={'person_id': author_id})
        committer_url = reverse('browse-person',
                                url_args={'person_id': committer_id})

        directory_url = reverse('browse-directory',
                                url_args={'sha1_git': dir_id})

        history_url = reverse('browse-revision-log',
                              url_args={'sha1_git': revision_id_test})

        resp = self.client.get(url)

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('browse/revision.html')
        self.assertContains(resp, '<a href="%s">%s</a>' %
                                  (author_url, author_name))
        self.assertContains(resp, '<a href="%s">%s</a>' %
                                  (committer_url, committer_name))
        self.assertContains(resp, directory_url)
        self.assertContains(resp, history_url)

        for parent in revision_metadata_test['parents']:
            parent_url = reverse('browse-revision',
                                 url_args={'sha1_git': parent})
            self.assertContains(resp, '<a href="%s">%s</a>' %
                                (parent_url, parent))

        author_date = revision_metadata_test['date']
        committer_date = revision_metadata_test['committer_date']

        message_lines = revision_metadata_test['message'].split('\n')

        self.assertContains(resp, format_utc_iso_date(author_date))
        self.assertContains(resp, format_utc_iso_date(committer_date))
        self.assertContains(resp, message_lines[0])
        self.assertContains(resp, '\n'.join(message_lines[1:]))

        origin_info = {
            'id': '7416001',
            'type': 'git',
            'url': 'https://github.com/webpack/webpack'
        }

        mock_service_utils.lookup_origin.return_value = origin_info
        mock_get_origin_visits.return_value = stub_origin_visits
        mock_get_origin_visit_snapshot.return_value = stub_origin_snapshot
        mock_service_utils.lookup_snapshot_size.return_value = {
            'revision': len(stub_origin_snapshot[0]),
            'release': len(stub_origin_snapshot[1])
        }

        origin_directory_url = reverse('browse-origin-directory',
                                       url_args={'origin_url': origin_info['url']},
                                       query_params={'revision': revision_id_test})

        origin_revision_log_url = reverse('browse-origin-log',
                                          url_args={'origin_url': origin_info['url']},
                                          query_params={'revision': revision_id_test})

        url = reverse('browse-revision',
                      url_args={'sha1_git': revision_id_test},
                      query_params={'origin': origin_info['url']})

        resp = self.client.get(url)

        self.assertContains(resp, origin_directory_url)

        self.assertContains(resp, origin_revision_log_url)

        for parent in revision_metadata_test['parents']:
            parent_url = reverse('browse-revision',
                                 url_args={'sha1_git': parent},
                                 query_params={'origin': origin_info['url']})
            self.assertContains(resp, '<a href="%s">%s</a>' %
                                (parent_url, parent))

        self.assertContains(resp, 'vault-cook-directory')
        self.assertContains(resp, 'vault-cook-revision')

        swh_rev_id = get_swh_persistent_id('revision', revision_id_test)
        swh_rev_id_url = reverse('browse-swh-id',
                                 url_args={'swh_id': swh_rev_id})
        self.assertContains(resp, swh_rev_id)
        self.assertContains(resp, swh_rev_id_url)

        swh_dir_id = get_swh_persistent_id('directory', dir_id)
        swh_dir_id_url = reverse('browse-swh-id',
                                 url_args={'swh_id': swh_dir_id})
        self.assertContains(resp, swh_dir_id)
        self.assertContains(resp, swh_dir_id_url)

    @patch('swh.web.browse.views.revision.service')
    def test_revision_log_browse(self, mock_service):
        per_page = 10

        revision_history_log_test_sorted = \
            sorted(revision_history_log_test,
                   key=lambda rev: -parse_timestamp(rev['committer_date']).timestamp())

        mock_revs_walker = MagicMock()
        mock_revs_walker.__iter__.return_value = revision_history_log_test_sorted
        mock_revs_walker.export_state.return_value = {}

        mock_service.get_revisions_walker.return_value = mock_revs_walker

        url = reverse('browse-revision-log',
                      url_args={'sha1_git': revision_id_test},
                      query_params={'per_page': per_page})

        resp = self.client.get(url)

        next_page_url = reverse('browse-revision-log',
                                url_args={'sha1_git': revision_id_test},
                                query_params={'offset': per_page,
                                              'per_page': per_page})

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('browse/revision-log.html')
        self.assertContains(resp, '<tr class="swh-revision-log-entry',
                            count=per_page)
        self.assertContains(resp, '<a class="page-link">Newer</a>')
        self.assertContains(resp, '<a class="page-link" href="%s">Older</a>' %
                            escape(next_page_url))

        for log in revision_history_log_test_sorted[:per_page]:
            author_url = reverse('browse-person',
                                 url_args={'person_id': log['author']['id']})
            revision_url = reverse('browse-revision',
                                   url_args={'sha1_git': log['id']})
            self.assertContains(resp, log['id'][:7])
            self.assertContains(resp, log['author']['name'])
            self.assertContains(resp, format_utc_iso_date(log['date']))
            self.assertContains(resp, escape(log['message']))
            self.assertContains(resp, format_utc_iso_date(log['committer_date']))
            self.assertContains(resp, revision_url)

        resp = self.client.get(next_page_url)

        prev_page_url = reverse('browse-revision-log',
                                url_args={'sha1_git': revision_id_test},
                                query_params={'per_page': per_page})
        next_page_url = reverse('browse-revision-log',
                                url_args={'sha1_git': revision_id_test},
                                query_params={'offset': 2 * per_page,
                                              'per_page': per_page})

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('browse/revision-log.html')
        self.assertContains(resp, '<tr class="swh-revision-log-entry',
                            count=per_page)
        self.assertContains(resp, '<a class="page-link" href="%s">Newer</a>' %
                            escape(prev_page_url))
        self.assertContains(resp, '<a class="page-link" href="%s">Older</a>' %
                            escape(next_page_url))

        resp = self.client.get(next_page_url)

        prev_page_url = reverse('browse-revision-log',
                                url_args={'sha1_git': revision_id_test},
                                query_params={'offset': per_page,
                                              'per_page': per_page})
        next_page_url = reverse('browse-revision-log',
                                url_args={'sha1_git': revision_id_test},
                                query_params={'offset': 3 * per_page,
                                              'per_page': per_page})

        self.assertEqual(resp.status_code, 200)
        self.assertTemplateUsed('browse/revision-log.html')
        self.assertContains(resp, '<tr class="swh-revision-log-entry',
                            count=per_page)
        self.assertContains(resp, '<a class="page-link" href="%s">Newer</a>' %
                            escape(prev_page_url))
        self.assertContains(resp, '<a class="page-link" href="%s">Older</a>' %
                            escape(next_page_url))

    @patch('swh.web.browse.utils.service')
    @patch('swh.web.browse.views.revision.service')
    def test_revision_request_errors(self, mock_service, mock_utils_service):
        mock_service.lookup_revision.side_effect = \
            NotFoundExc('Revision not found')
        url = reverse('browse-revision',
                      url_args={'sha1_git': revision_id_test})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertContains(resp, 'Revision not found', status_code=404)

        mock_service.get_revisions_walker.side_effect = \
            NotFoundExc('Revision not found')
        url = reverse('browse-revision-log',
                      url_args={'sha1_git': revision_id_test})
        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertContains(resp, 'Revision not found', status_code=404)

        url = reverse('browse-revision',
                      url_args={'sha1_git': revision_id_test},
                      query_params={'origin_type': 'git',
                                    'origin': 'https://github.com/foo/bar'})

        mock_service.lookup_revision.side_effect = None
        mock_utils_service.lookup_origin.side_effect = \
            NotFoundExc('Origin not found')

        resp = self.client.get(url)
        self.assertEqual(resp.status_code, 404)
        self.assertTemplateUsed('error.html')
        self.assertContains(resp, 'the origin mentioned in your request appears broken',
                            status_code=404)



# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

# flake8: noqa

revision_id_test = '7bc08e1aa0b08cb23e18715a32aa38517ad34672'

revision_metadata_test = \
{'id': '7bc08e1aa0b08cb23e18715a32aa38517ad34672',
 'type': 'git',
 'parents': ['bf3652b16b65c27db5243aa0d674e2de4a8ccde9',
             'a952bb99a6830804d06c5b8e04b75c66100fbae9'],
 'metadata': {},
 'committer': {'name': 'GitHub',
               'fullname': 'GitHub <noreply@github.com>',
               'id': 10932771,
               'email': 'noreply@github.com'},
 'directory': 'ae59ceecf46367e8e4ad800e231fc76adc3afffb',
 'date': '2017-05-04T13:27:13+02:00',
 'merge': True,
 'committer_date': '2017-05-04T13:27:13+02:00',
 'author': {'name': 'Tobias Koppers',
            'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
            'id': 141959,
            'email': 'tobias.koppers@googlemail.com'},
 'message': 'Merge pull request #4816 from webpack/bugfix/hoist-immutable-export\n\nhoist exports',
 'synthetic': False
}

revision_history_log_test = \
[{'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-05-04T13:27:13+02:00',
  'date': '2017-05-04T13:27:13+02:00',
  'directory': 'ae59ceecf46367e8e4ad800e231fc76adc3afffb',
  'id': '7bc08e1aa0b08cb23e18715a32aa38517ad34672',
  'merge': True,
  'message': 'Merge pull request #4816 from '
             'webpack/bugfix/hoist-immutable-export\n'
             '\n'
             'hoist exports',
  'metadata': {},
  'parents': ['bf3652b16b65c27db5243aa0d674e2de4a8ccde9',
              'a952bb99a6830804d06c5b8e04b75c66100fbae9'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-05-04T12:17:13+02:00',
  'date': '2017-05-04T12:17:13+02:00',
  'directory': '344f6b38f021fa0bbd60ca06fe1cbc61164e7abe',
  'id': 'bf3652b16b65c27db5243aa0d674e2de4a8ccde9',
  'merge': False,
  'message': '2.5.0\n',
  'metadata': {},
  'parents': ['cd1cd29fba46bd0133db0ca89acbe6c6c0240323'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-05-04T13:00:52+02:00',
  'date': '2017-05-04T12:56:31+02:00',
  'directory': 'ae59ceecf46367e8e4ad800e231fc76adc3afffb',
  'id': 'a952bb99a6830804d06c5b8e04b75c66100fbae9',
  'merge': False,
  'message': 'change some magic numbers to hoist exports\n\nfixes #4753\n',
  'metadata': {},
  'parents': ['bf3652b16b65c27db5243aa0d674e2de4a8ccde9'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-05-04T11:41:43+02:00',
  'date': '2017-05-04T11:41:43+02:00',
  'directory': 'fbc01dab452f80bf49d554cc920979a66839707f',
  'id': 'cd1cd29fba46bd0133db0ca89acbe6c6c0240323',
  'merge': True,
  'message': 'Merge pull request #4815 from '
             'webpack/bugfix/extract-async-initial\n'
             '\n'
             "CommonsChunkPlugin in async mode doesn't select initial chunks",
  'metadata': {},
  'parents': ['8bab88c50fcb87c749c244b9ab28a1fb7e173bea',
              'b45588bc1197abbc309eb3705a4bf89960b001ae'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-05-04T10:50:03+02:00',
  'date': '2017-05-04T10:50:03+02:00',
  'directory': '74ed598c6b831fe7f68697f6ed761660a842b973',
  'id': '8bab88c50fcb87c749c244b9ab28a1fb7e173bea',
  'merge': True,
  'message': 'Merge pull request #4814 from webpack/test/move-entry\n'
             '\n'
             'add testcase for moving entry modules into the commons chunk',
  'metadata': {},
  'parents': ['85dc98f17aa39d5d3337e3791bf25634a1f7e445',
              'a244879a07e04e6b5951520ca3cd80c3ef160f8e'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-05-04T10:50:46+02:00',
  'date': '2017-05-04T10:50:46+02:00',
  'directory': '735b9f6d54d62470944c4743d52931b308a9072c',
  'id': 'b45588bc1197abbc309eb3705a4bf89960b001ae',
  'merge': False,
  'message': "CommonsChunkPlugin in async mode doesn't select initial chunks\n"
             '\n'
             'fixes #4795\n',
  'metadata': {},
  'parents': ['c91ba4949503de5ca9d98c98188c2654b095f2cb'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-05-04T08:04:41+02:00',
  'date': '2017-05-04T08:04:41+02:00',
  'directory': 'c3c24aeafabe1d441c37a2769e9b65ae10075925',
  'id': '85dc98f17aa39d5d3337e3791bf25634a1f7e445',
  'merge': True,
  'message': 'Merge pull request #4813 from JLHwung/perf/date-now\n'
             '\n'
             'Perf/use Date.now() instead of +new Date()/new Date().getTime()',
  'metadata': {},
  'parents': ['c91ba4949503de5ca9d98c98188c2654b095f2cb',
              '6afc397b99aef338a9d66add4488ce03ea3f7a43'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-05-04T10:01:04+02:00',
  'date': '2017-05-04T10:01:04+02:00',
  'directory': '4de09be1628ed81def03f78d8d832c93efdf0af4',
  'id': 'a244879a07e04e6b5951520ca3cd80c3ef160f8e',
  'merge': False,
  'message': 'add testcase for moving entry modules into the commons chunk\n'
             '#4795\n',
  'metadata': {},
  'parents': ['c91ba4949503de5ca9d98c98188c2654b095f2cb'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-30T09:31:17+02:00',
  'date': '2017-04-30T09:31:17+02:00',
  'directory': '16c9d449871efccef7a0f53b29bd6218cc769e30',
  'id': 'c91ba4949503de5ca9d98c98188c2654b095f2cb',
  'merge': True,
  'message': 'Merge pull request #4791 from deificx/master\n'
             '\n'
             'add option to lib/Stats.js to disable stack trace on errors and '
             'warnings',
  'metadata': {},
  'parents': ['94ba75f7940836390c041846f2c334929ee14332',
              '84ea1ffd3d0892f3356ac0494947bbc2a0e39d51'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'i@jhuang.me',
             'fullname': 'J Huang <i@jhuang.me>',
             'id': 12072398,
             'name': 'J Huang'},
  'committer': {'email': 'i@jhuang.me',
                'fullname': 'J Huang <i@jhuang.me>',
                'id': 12072398,
                'name': 'J Huang'},
  'committer_date': '2017-05-04T10:19:54+08:00',
  'date': '2017-05-04T10:19:54+08:00',
  'directory': 'c3c24aeafabe1d441c37a2769e9b65ae10075925',
  'id': '6afc397b99aef338a9d66add4488ce03ea3f7a43',
  'merge': False,
  'message': 'perf: use Date.now() instead of new Date().getTime()\n'
             '\n'
             'new Date().getTime() is 2x slower than Date.now(), see '
             'https://jsperf.com/new-date-vs-date-now-vs-performance-now/6\n',
  'metadata': {},
  'parents': ['94d0641ba40d65d5fcbd64414b6ea9f8a2589538'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-30T09:30:03+02:00',
  'date': '2017-04-30T09:30:03+02:00',
  'directory': 'f78fe6939706d761f0bc60f9d566bd414ef5c113',
  'id': '94ba75f7940836390c041846f2c334929ee14332',
  'merge': True,
  'message': 'Merge pull request #4794 from '
             'ndresx/disable-manifest-json-pretty-print\n'
             '\n'
             'Disable manifest.json pretty print',
  'metadata': {},
  'parents': ['24ef6ea1b56b358caeb4b07476a909f4f86c2d8a',
              'de87f93c1b050db59ffbeff3aed4ac3b3eb57da3'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-29T20:53:42+02:00',
  'date': '2017-04-29T20:52:43+02:00',
  'directory': '68fcb2d3c60aa32b708ce9ca9fa5c4b1f7a2532d',
  'id': '84ea1ffd3d0892f3356ac0494947bbc2a0e39d51',
  'merge': False,
  'message': 'added error to stats.moduleTrace test name to trigger test cases '
             'corretly\n',
  'metadata': {},
  'parents': ['8ad4386bdf9cd607310a4255dff8cb9ccd12afad'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'i@jhuang.me',
             'fullname': 'J Huang <i@jhuang.me>',
             'id': 12072398,
             'name': 'J Huang'},
  'committer': {'email': 'i@jhuang.me',
                'fullname': 'J Huang <i@jhuang.me>',
                'id': 12072398,
                'name': 'J Huang'},
  'committer_date': '2017-05-04T10:19:10+08:00',
  'date': '2017-05-04T10:17:23+08:00',
  'directory': '41965bffa3f73c8928beb2a6580c253adb9bd5cf',
  'id': '94d0641ba40d65d5fcbd64414b6ea9f8a2589538',
  'merge': False,
  'message': 'perf: use Date.now() instead of +new Date()\n'
             '\n'
             '+new Date() is 2x slower than Date.now(), see '
             'https://jsperf.com/new-date-vs-date-now-vs-performance-now/6\n',
  'metadata': {},
  'parents': ['c91ba4949503de5ca9d98c98188c2654b095f2cb'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-26T09:05:56+02:00',
  'date': '2017-04-26T09:05:56+02:00',
  'directory': 'aa78629e4aefc836b5a967db57cac29f79fea074',
  'id': '24ef6ea1b56b358caeb4b07476a909f4f86c2d8a',
  'merge': True,
  'message': 'Merge pull request #4779 from ndresx/ignoreplugin-typo-fix\n'
             '\n'
             'Fix typo in IgnorePlugin',
  'metadata': {},
  'parents': ['34315db3ffe20b3d7e9a50885ea623fc90ac0861',
              'aa172f0a359ab3c0b0987101c64c8b41b07d581f'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'ndresx@gmail.com',
             'fullname': 'Martin Veith <ndresx@gmail.com>',
             'id': 14855385,
             'name': 'Martin Veith'},
  'committer': {'email': 'ndresx@gmail.com',
                'fullname': 'Martin Veith <ndresx@gmail.com>',
                'id': 14855385,
                'name': 'Martin Veith'},
  'committer_date': '2017-04-28T21:34:38+02:00',
  'date': '2017-04-28T21:34:38+02:00',
  'directory': 'f78fe6939706d761f0bc60f9d566bd414ef5c113',
  'id': 'de87f93c1b050db59ffbeff3aed4ac3b3eb57da3',
  'merge': False,
  'message': 'Disable manifest.json pretty print\n',
  'metadata': {},
  'parents': ['24ef6ea1b56b358caeb4b07476a909f4f86c2d8a'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-29T19:43:45+02:00',
  'date': '2017-04-29T19:43:45+02:00',
  'directory': '893a65a9ad785d719c44a3e764d150bb47cba6c1',
  'id': '8ad4386bdf9cd607310a4255dff8cb9ccd12afad',
  'merge': False,
  'message': 'test cases for stats.moduleTrace option\n',
  'metadata': {},
  'parents': ['958156ae4201e75f8cb04e306d2a38b94d2a8a1d'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T14:25:42+02:00',
  'date': '2017-04-23T14:25:42+02:00',
  'directory': 'f511a1febf3d6b9b788080f3eb69ff3cb963758f',
  'id': '34315db3ffe20b3d7e9a50885ea623fc90ac0861',
  'merge': True,
  'message': 'Merge pull request #3875 from webpack/test/circleci\n'
             '\n'
             'Update circle.yml',
  'metadata': {},
  'parents': ['6a26e9ba7f7f1be7e76054f219bf1e094f2c3264',
              '0f91f949e2d41ef5cb92493bcc6c1fa7578ac27d'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'ndresx@gmail.com',
             'fullname': 'Martin Veith <ndresx@gmail.com>',
             'id': 14855385,
             'name': 'Martin Veith'},
  'committer': {'email': 'ndresx@gmail.com',
                'fullname': 'Martin Veith <ndresx@gmail.com>',
                'id': 14855385,
                'name': 'Martin Veith'},
  'committer_date': '2017-04-25T22:07:43+02:00',
  'date': '2017-04-25T22:07:43+02:00',
  'directory': 'aa78629e4aefc836b5a967db57cac29f79fea074',
  'id': 'aa172f0a359ab3c0b0987101c64c8b41b07d581f',
  'merge': False,
  'message': 'Fix typo in IgnorePlugin\n',
  'metadata': {},
  'parents': ['34315db3ffe20b3d7e9a50885ea623fc90ac0861'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-29T19:43:15+02:00',
  'date': '2017-04-29T19:43:15+02:00',
  'directory': '9d7347c42e29afda2849996ca6b61cdc7ea1b5a1',
  'id': '958156ae4201e75f8cb04e306d2a38b94d2a8a1d',
  'merge': False,
  'message': 'moduleTrace added to webpackOptionsSchema.json\n',
  'metadata': {},
  'parents': ['41310135bb4e30a0a6f71eddd3d74419ec8512a7'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:57:02+02:00',
  'date': '2017-04-23T12:57:02+02:00',
  'directory': '71848e66952829c98ec4e57e2338be986390b850',
  'id': '6a26e9ba7f7f1be7e76054f219bf1e094f2c3264',
  'merge': True,
  'message': 'Merge pull request #4693 from '
             'Travmatth/fix-4252-BannerPlugin-placeholder\n'
             '\n'
             'Fix 4252 banner plugin placeholder',
  'metadata': {},
  'parents': ['53bb15b1ed64f8636036f773100d502909bd1e6b',
              '08eca2fb4517bb2a14e21fa46e3291878c79be0e'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-04-23T14:02:06+02:00',
  'date': '2017-04-23T12:50:01+02:00',
  'directory': '29cf488a00126c0e45c78b92f97cdbc733f4d07e',
  'id': '0f91f949e2d41ef5cb92493bcc6c1fa7578ac27d',
  'merge': False,
  'message': 'improve circleci build\n',
  'metadata': {},
  'parents': ['53bb15b1ed64f8636036f773100d502909bd1e6b'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-28T09:20:14+02:00',
  'date': '2017-04-28T09:20:14+02:00',
  'directory': 'f8b679bd62a33880d7f39f70233c3c14aead7c22',
  'id': '41310135bb4e30a0a6f71eddd3d74419ec8512a7',
  'merge': False,
  'message': 'rename stats.stackTrace to stats.moduleTrace\n',
  'metadata': {},
  'parents': ['7e4310a1759bd35834290e4c4af207ce9b05ffd9'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:10:20+02:00',
  'date': '2017-04-23T12:10:20+02:00',
  'directory': '1887ba14662a7e45567fa6daa39ab72975b429aa',
  'id': '53bb15b1ed64f8636036f773100d502909bd1e6b',
  'merge': True,
  'message': 'Merge pull request #3934 from '
             'timse/refactor-watching-in-compiler\n'
             '\n'
             'Refactor _done of Watching in compiler',
  'metadata': {},
  'parents': ['b67d61abd6dd878f7a9aec4d10689c8b1678e28a',
              'ab30c6b1c0100634ad46bcaf47ff12963e22aac9'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:00:53+02:00',
  'date': '2017-04-23T12:00:53+02:00',
  'directory': '4da5fda41a4b5cdbcf1fbb2604e9696c67106271',
  'id': '08eca2fb4517bb2a14e21fa46e3291878c79be0e',
  'merge': False,
  'message': 'spacing',
  'metadata': {},
  'parents': ['7061c2c76b984e9c7ab893bf4b14b4b102b5fa0d'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-27T10:22:20+02:00',
  'date': '2017-04-27T10:22:20+02:00',
  'directory': '21f3f99258fad8513cbdd40f549cb067362add27',
  'id': '7e4310a1759bd35834290e4c4af207ce9b05ffd9',
  'merge': False,
  'message': 'add option to lib/Stats.js to disable stack trace on errors and '
             'warnings\n',
  'metadata': {},
  'parents': ['24ef6ea1b56b358caeb4b07476a909f4f86c2d8a'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:04:28+02:00',
  'date': '2017-04-23T12:04:28+02:00',
  'directory': 'effe11da8d44198a760cab867351f66d4b0da7d1',
  'id': 'b67d61abd6dd878f7a9aec4d10689c8b1678e28a',
  'merge': True,
  'message': 'Merge pull request #4755 from aretecode/lint-update\n'
             '\n'
             'update node linting',
  'metadata': {},
  'parents': ['a2ec4c8cea281715d3043a6dcb4198be039cfb54',
              '5c423d6feb6521c9b5274ba342b7621c453f35eb'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tim.sebastian@gmail.com',
             'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
             'id': 3654888,
             'name': 'Tim Sebastian'},
  'committer': {'email': 'tim.sebastian@gmail.com',
                'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
                'id': 3654888,
                'name': 'Tim Sebastian'},
  'committer_date': '2017-04-08T09:13:38+10:00',
  'date': '2017-04-08T09:13:38+10:00',
  'directory': '20805e5ff5776062b9108138c4dfce8124185f5a',
  'id': 'ab30c6b1c0100634ad46bcaf47ff12963e22aac9',
  'merge': False,
  'message': 'pass stats to error handler again, to prevent breaking change\n',
  'metadata': {},
  'parents': ['b863851ce969317688799b754131e3546206c7ad'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'trav.matth@gmail.com',
             'fullname': 'Travis Matthews <trav.matth@gmail.com>',
             'id': 439443,
             'name': 'Travis Matthews'},
  'committer': {'email': 'trav.matth@gmail.com',
                'fullname': 'Travis Matthews <trav.matth@gmail.com>',
                'id': 439443,
                'name': 'Travis Matthews'},
  'committer_date': '2017-04-19T09:46:34-04:00',
  'date': '2017-04-19T09:46:34-04:00',
  'directory': '3c9ea2ac7fec857634b7b46dc08b52a5670106c5',
  'id': '7061c2c76b984e9c7ab893bf4b14b4b102b5fa0d',
  'merge': False,
  'message': 'added file test, remove unused [basename]\n',
  'metadata': {},
  'parents': ['527f9434f74e63ae76ecbf18a4710589667dc731'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:02:55+02:00',
  'date': '2017-04-23T12:02:55+02:00',
  'directory': '5185e53216f3a836598f946f7302155f5bb50d02',
  'id': 'a2ec4c8cea281715d3043a6dcb4198be039cfb54',
  'merge': True,
  'message': 'Merge pull request #4722 from '
             'willmendesneto/refactor-format-location\n'
             '\n'
             'refactor(formatLocation): upgrade to ES6',
  'metadata': {},
  'parents': ['1c5f3bf59f8a1e06bbe83b7459362f69bb5b7460',
              'c826edde1be8af20a02166f792f79bbf2a231c4e'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-22T19:02:01+02:00',
  'date': '2017-04-22T19:02:01+02:00',
  'directory': '461c61e52405649d272d7e79b8a1abe27e5b33b2',
  'id': '5c423d6feb6521c9b5274ba342b7621c453f35eb',
  'merge': False,
  'message': 'change supported node.js version to 4',
  'metadata': {},
  'parents': ['219a95dddca951737d4b947fc2d59e3a2b5cbb23'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tim.sebastian@gmail.com',
             'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
             'id': 3654888,
             'name': 'Tim Sebastian'},
  'committer': {'email': 'tim.sebastian@gmail.com',
                'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
                'id': 3654888,
                'name': 'Tim Sebastian'},
  'committer_date': '2017-04-06T21:52:06+10:00',
  'date': '2017-04-06T21:52:06+10:00',
  'directory': '803b83492f4be6b472720dfd5dd0f45675213ac2',
  'id': 'b863851ce969317688799b754131e3546206c7ad',
  'merge': False,
  'message': 'add getStats again - remove breaking change\n',
  'metadata': {},
  'parents': ['2a1bcff9e6999d471747ab3575d82c33a0e5e082'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'trav.matth@gmail.com',
             'fullname': 'Travis Matthews <trav.matth@gmail.com>',
             'id': 439443,
             'name': 'Travis Matthews'},
  'committer': {'email': 'trav.matth@gmail.com',
                'fullname': 'Travis Matthews <trav.matth@gmail.com>',
                'id': 439443,
                'name': 'Travis Matthews'},
  'committer_date': '2017-04-19T09:28:25-04:00',
  'date': '2017-04-19T09:28:25-04:00',
  'directory': 'aafcbc664c51bf17dcfde6b7112efe95331c6fcd',
  'id': '527f9434f74e63ae76ecbf18a4710589667dc731',
  'merge': False,
  'message': 'removed single quotes\n',
  'metadata': {},
  'parents': ['c68bd16b265b6d5f8478e5728ea077ead90e8a37'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T11:49:28+02:00',
  'date': '2017-04-23T11:49:28+02:00',
  'directory': 'cf61c993ed0335a88d7975aba07ef743eaa7b845',
  'id': '1c5f3bf59f8a1e06bbe83b7459362f69bb5b7460',
  'merge': True,
  'message': 'Merge pull request #4748 from '
             'aretecode/examples-dll-readme-update2\n'
             '\n'
             'examples/dll readme updates',
  'metadata': {},
  'parents': ['cbd493904f222abac1d33918b4686b0bc4de3a42',
              'f223aecd05b47b9cfff34609f730c81d03034514'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'willmendesneto@gmail.com',
             'fullname': 'Will Mendes <willmendesneto@gmail.com>',
             'id': 93385,
             'name': 'Will Mendes'},
  'committer': {'email': 'willmendesneto@gmail.com',
                'fullname': 'Will Mendes <willmendesneto@gmail.com>',
                'id': 93385,
                'name': 'Will Mendes'},
  'committer_date': '2017-04-17T22:00:47+10:00',
  'date': '2017-04-17T22:00:47+10:00',
  'directory': '817b5de0d36aae8f3243b472e19b2fae1adf9173',
  'id': 'c826edde1be8af20a02166f792f79bbf2a231c4e',
  'merge': False,
  'message': 'refactor(formatLocation): upgrade to ES6\n',
  'metadata': {},
  'parents': ['82ddd16080663e3773bdac2a62082991acb8637e'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'aretecode@gmail.com',
             'fullname': 'James <aretecode@gmail.com>',
             'id': 11020868,
             'name': 'James'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-19T23:53:44-07:00',
  'date': '2017-04-19T23:53:44-07:00',
  'directory': 'fa17b78edd5fd19a589663489e91b14e2c6337c0',
  'id': '219a95dddca951737d4b947fc2d59e3a2b5cbb23',
  'merge': False,
  'message': 'lint autofix the eslint file',
  'metadata': {},
  'parents': ['cf666188298dfe75cc343018bf35b1c753fb6636'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tim.sebastian@gmail.com',
             'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
             'id': 3654888,
             'name': 'Tim Sebastian'},
  'committer': {'email': 'tim.sebastian@gmail.com',
                'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
                'id': 3654888,
                'name': 'Tim Sebastian'},
  'committer_date': '2017-04-06T21:46:07+10:00',
  'date': '2017-01-12T22:33:03+11:00',
  'directory': '7c6b85c086b56f11cf29bc3d13836145dc225407',
  'id': '2a1bcff9e6999d471747ab3575d82c33a0e5e082',
  'merge': False,
  'message': 'refactor the _done method of Watching\n'
             '\n'
             '- we can safely ignore the "else" cases of not having an error '
             'as _done() is only called without arguments if "this.invalid is '
             'true"\n'
             '- if we get passed the point of `this.invalid` either `err` or '
             '`compilation` are !!always!! set. therefore later checks can '
             'again be ignored\n'
             '- early return in error case\n'
             '- ignore `this.error` if we make it passed the error as it will '
             'be unset at this point.\n'
             '- remove the setting of `this.error` or `this.stats` as the only '
             'use is inside this method and only allow weird behaviour if '
             'someone set them from outside\n',
  'metadata': {},
  'parents': ['8165164d3e50fef3252d7c274cef1c0b595c6992'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'trav.matth@gmail.com',
             'fullname': 'Travis Matthews <trav.matth@gmail.com>',
             'id': 439443,
             'name': 'Travis Matthews'},
  'committer': {'email': 'trav.matth@gmail.com',
                'fullname': 'Travis Matthews <trav.matth@gmail.com>',
                'id': 439443,
                'name': 'Travis Matthews'},
  'committer_date': '2017-04-19T08:41:00-04:00',
  'date': '2017-04-19T08:41:00-04:00',
  'directory': '34bfecaa341626e1c60d1d2a1f697fb1f74f8bcd',
  'id': 'c68bd16b265b6d5f8478e5728ea077ead90e8a37',
  'merge': False,
  'message': 'remove comment\n',
  'metadata': {},
  'parents': ['f2e30693ac0e3a9a0fe6a01a89fe6072dee7f405'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-22T12:45:04+02:00',
  'date': '2017-04-22T12:45:04+02:00',
  'directory': '412c6f5c00ce7d02efcdd6f23160a36eb4e5a1f4',
  'id': 'cbd493904f222abac1d33918b4686b0bc4de3a42',
  'merge': True,
  'message': 'Merge pull request #4768 from xizhao/patch-1\n'
             '\n'
             'Add license scan report and status',
  'metadata': {},
  'parents': ['d7f30392ddda27613e0ea05cb60ec985b4f75e5c',
              'ff211108d888908d41470cea6187133ccdb56e87'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'aretecode@gmail.com',
             'fullname': 'Arete Code <aretecode@gmail.com>',
             'id': 8227899,
             'name': 'Arete Code'},
  'committer': {'email': 'aretecode@gmail.com',
                'fullname': 'Arete Code <aretecode@gmail.com>',
                'id': 8227899,
                'name': 'Arete Code'},
  'committer_date': '2017-04-21T15:48:06-07:00',
  'date': '2017-04-21T15:48:06-07:00',
  'directory': '9d69f66cf3d2fa5119910bfd6a16f16e996795cc',
  'id': 'f223aecd05b47b9cfff34609f730c81d03034514',
  'merge': False,
  'message': 'rebase for semicolon\n',
  'metadata': {},
  'parents': ['914fe2c923533d43f5edff54b1704bbd3dd407bc'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-15T19:26:54+02:00',
  'date': '2017-04-15T19:26:54+02:00',
  'directory': 'c2da553b359190d3dc4266d77e311bd002d7ccd2',
  'id': '82ddd16080663e3773bdac2a62082991acb8637e',
  'merge': True,
  'message': 'Merge pull request #4717 from STRML/fix/hashSalt\n'
             '\n'
             'fix missing `hashSalt` from options schema',
  'metadata': {},
  'parents': ['bd753567da1248624beaaea14af31d6dbe303411',
              '805c9fadf05c00f3df16d1faf4d3f608b78e3b59'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'aretecode@gmail.com',
             'fullname': 'Arete Code <aretecode@gmail.com>',
             'id': 8227899,
             'name': 'Arete Code'},
  'committer': {'email': 'aretecode@gmail.com',
                'fullname': 'Arete Code <aretecode@gmail.com>',
                'id': 8227899,
                'name': 'Arete Code'},
  'committer_date': '2017-04-19T19:05:28-07:00',
  'date': '2017-04-19T19:05:28-07:00',
  'directory': 'dd7dc17a8a28b6bb415b1f6980de6fcd3715847c',
  'id': 'cf666188298dfe75cc343018bf35b1c753fb6636',
  'merge': False,
  'message': '👕 update node linting\n'
             '\n'
             'update linting for destructuring and latest node\n'
             '\n'
             'uses .js rather than .eslintrc file\n'
             '\n'
             'pulls in rules from\n'
             'https://github.com/webpack/webpack-cli/pull/46/files#diff-df39304d828831c44a2b9f38cd45289cR40\n'
             '\n'
             'adds spacing for this\n'
             '<img width="495" alt="screen shot 2017-04-19 at 7 03 16 pm" '
             'src="https://cloud.githubusercontent.com/assets/4022631/25210014/eabff164-2532-11e7-89a3-837fde6c7101.png">\n',
  'metadata': {},
  'parents': ['8aa8a7b63fed3c7909a6d4f15159c036a0561d64'],
  'synthetic': False,
  'type': 'git'},
 {'author': {
    'email': 'tobias.koppers@googlemail.com',
    'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
    'id': 141959,
    'name': 'Tobias Koppers'
  },
  'committer': {
    'email': 'noreply@github.com',
    'fullname': 'GitHub <noreply@github.com>',
    'id': 10932771,
    'name': 'GitHub'
  },
  'committer_date': '2017-04-21T19:00:50+02:00',
  'committer_url': '/api/1/person/10932771/',
  'date': '2017-04-21T19:00:50+02:00',
  'directory': '52cf6f28b1dbfe98f485ea78ae03942f55cd8fa0',
  'id': 'd7f30392ddda27613e0ea05cb60ec985b4f75e5c',
  'merge': True,
  'message': 'Merge pull request #4729 from simon04/provide-plugin-es2015\n\nProvidePlugin: add test case for ES2015 modules',
  'metadata': {},
  'parents': ['88f37348e7de240d794713c0c38170f17a0a8c0e',
              'd0bbf967fb51c031e16c5dfe040afce9a4113b5b'],
  'synthetic': False,
  'type': 'git'},
 {'author': {
    'email': 'kevin@fossa.io',
    'fullname': 'Kevin Wang <kevin@fossa.io>',
    'id': 14855611,
    'name': 'Kevin Wang'
  },
  'committer': {
    'email': 'noreply@github.com',
    'fullname': 'GitHub <noreply@github.com>',
    'id': 10932771,
    'name': 'GitHub'
  },
  'committer_date': '2017-04-21T18:28:41-07:00',
  'date': '2017-04-21T18:28:41-07:00',
  'directory': '412c6f5c00ce7d02efcdd6f23160a36eb4e5a1f4',
  'id': 'ff211108d888908d41470cea6187133ccdb56e87',
  'merge': False,
  'message': 'Add license scan report and status',
  'metadata': {},
  'parents': ['d7f30392ddda27613e0ea05cb60ec985b4f75e5c'],
  'synthetic': False,
  'type': 'git'},
]